# Updated Animation Starter Code

from tkinter import *

####################################
# customize these functions
####################################
import random

# Draws a bouncing ball which can be paused

def init(data):
    data.ballSize = 50
    data.ballX = random.randint(data.ballSize,data.width-data.ballSize)
    data.ballY = random.randint(data.ballSize,data.height-data.ballSize)
    data.dx = random.randint(1,4)
    data.dy = random.randint(1,4)
    data.isPaused = False
    data.timerDelay = 10

def keyPressed(event, data):
    if (event.char == "p"):
        data.isPaused = not data.isPaused
    elif (event.char == "s"):
        doStep(data)

def timerFired(data):
    if (not data.isPaused):
        doStep(data)

def doStep(data):
    # Move horizontally
    data.ballX += data.dx
    # Check if the ball has gone out of bounds
    if data.ballX-data.ballSize < 0 or data.ballX + data.ballSize > data.width:
        # if so, reverse!
        data.dx = - data.dx
        # also, undo the move to avoid moving offscreen
        data.ballX += data.dx
    
    # Move vertically the same way
    data.ballY += data.dy
    if data.ballY-data.ballSize < 0 or data.ballY + data.ballSize > data.height:
        data.dy = - data.dy
        data.ballY += data.dy

def redrawAll(canvas, data):
    # draw the ball
    canvas.create_oval(data.ballX-data.ballSize,
                            data.ballY-data.ballSize,
                            data.ballX + data.ballSize,
                            data.ballY + data.ballSize,
                            fill="red")
    # draw the text
    canvas.create_text(data.width/2, 20,
                       text="Pressing 'p' pauses/unpauses timer")
    canvas.create_text(data.width/2, 40,
                       text="Pressing 's' steps the timer once")

####################################
# use the run function as-is
####################################

def run(width=300, height=300):
    def redrawAllWrapper(canvas, data):
        canvas.delete(ALL)
        canvas.create_rectangle(0, 0, data.width, data.height,
                                fill='white', width=0)
        redrawAll(canvas, data)
        canvas.update()    

    def mousePressedWrapper(event, canvas, data):
        mousePressed(event, data)
        redrawAllWrapper(canvas, data)

    def keyPressedWrapper(event, canvas, data):
        keyPressed(event, data)
        redrawAllWrapper(canvas, data)

    def timerFiredWrapper(canvas, data):
        timerFired(data)
        redrawAllWrapper(canvas, data)
        # pause, then call timerFired again
        canvas.after(data.timerDelay, timerFiredWrapper, canvas, data)
    # Set up data and call init
    class Struct(object): pass
    data = Struct()
    data.width = width
    data.height = height
    data.timerDelay = 100 # milliseconds
    root = Tk()
    root.resizable(width=False, height=False) # prevents resizing window
    init(data)
    # create the root and the canvas
    canvas = Canvas(root, width=data.width, height=data.height)
    canvas.configure(bd=0, highlightthickness=0)
    canvas.pack()
    # set up events
    root.bind("<Button-1>", lambda event:
                            mousePressedWrapper(event, canvas, data))
    root.bind("<Key>", lambda event:
                            keyPressedWrapper(event, canvas, data))
    timerFiredWrapper(canvas, data)
    # and launch the app
    root.mainloop()  # blocks until window is closed
    print("bye!")

run(800, 800)